package sun.awt.image;

import java.security.AccessController;
import java.security.PrivilegedAction;
import sun.awt.AppContext;

class ImageFetcher extends Thread
{
    static final int HIGH_PRIORITY = 8;
    static final int LOW_PRIORITY = 3;
    static final int ANIM_PRIORITY = 2;
    static final int TIMEOUT = 5000;
    
    private ImageFetcher(ThreadGroup group, int i) {
        super(group, "Image Fetcher " + i);
        this.setDaemon(true);
    }
    
    public static boolean add(ImageFetchable obj) {
        FetcherInfo fetcherInfo = FetcherInfo.getFetcherInfo();
        synchronized (fetcherInfo.waitList) {
            if (!fetcherInfo.waitList.contains(obj)) {
                fetcherInfo.waitList.addElement(obj);
                if (fetcherInfo.numWaiting == 0 && fetcherInfo.numFetchers < fetcherInfo.fetchers.length) {
                    createFetchers(fetcherInfo);
                }
                if (fetcherInfo.numFetchers > 0) {
                    fetcherInfo.waitList.notify();
                }
                else {
                    fetcherInfo.waitList.removeElement(obj);
                    return false;
                }
            }
        }
        return true;
    }
    
    public static void remove(ImageFetchable imageFetchable) {
        FetcherInfo fetcherInfo = FetcherInfo.getFetcherInfo();
        synchronized (fetcherInfo.waitList) {
            if (fetcherInfo.waitList.contains(imageFetchable)) {
                fetcherInfo.waitList.removeElement(imageFetchable);
            }
        }
    }
    
    public static boolean isFetcher(Thread thread) {
        FetcherInfo fetcherInfo = FetcherInfo.getFetcherInfo();
        synchronized (fetcherInfo.waitList) {
            int i = 0;
            while (i < fetcherInfo.fetchers.length) {
                if (fetcherInfo.fetchers[i] == thread) {
                    return true;
                }
                ++i;
            }
        }
        return false;
    }
    
    public static boolean amFetcher() {
        return isFetcher(Thread.currentThread());
    }
    
    private static ImageFetchable nextImage() {
        FetcherInfo fetcherInfo = FetcherInfo.getFetcherInfo();
        synchronized (fetcherInfo.waitList) {
            ImageFetchable obj = null;
            long n = System.currentTimeMillis() + 5000L;
            while (obj == null) {
                while (fetcherInfo.waitList.size() == 0) {
                    long currentTimeMillis = System.currentTimeMillis();
                    if (currentTimeMillis >= n) {
                        return null;
                    }
                    try {
                        ++fetcherInfo.numWaiting;
                        fetcherInfo.waitList.wait(n - currentTimeMillis);
                    }
                    catch (InterruptedException ex) {
                        __Null _Null = null;
                        --fetcherInfo.numWaiting;
                        return _Null;
                    }
                    finally {
                        --fetcherInfo.numWaiting;
                    }
                }
                obj = (ImageFetchable)fetcherInfo.waitList.elementAt(0);
                fetcherInfo.waitList.removeElement(obj);
            }
            return obj;
        }
    }
    
    @Override
    public void run() {
        FetcherInfo fetcherInfo = FetcherInfo.getFetcherInfo();
        try {
            this.fetchloop();
            synchronized (fetcherInfo.waitList) {
                Thread currentThread = Thread.currentThread();
                for (int i = 0; i < fetcherInfo.fetchers.length; ++i) {
                    if (fetcherInfo.fetchers[i] == currentThread) {
                        fetcherInfo.fetchers[i] = null;
                        --fetcherInfo.numFetchers;
                    }
                }
            }
        }
        catch (Exception ex) {
            ex.printStackTrace();
            synchronized (fetcherInfo.waitList) {
                Thread currentThread2 = Thread.currentThread();
                for (int j = 0; j < fetcherInfo.fetchers.length; ++j) {
                    if (fetcherInfo.fetchers[j] == currentThread2) {
                        fetcherInfo.fetchers[j] = null;
                        --fetcherInfo.numFetchers;
                    }
                }
            }
        }
        finally {
            synchronized (fetcherInfo.waitList) {
                Thread currentThread3 = Thread.currentThread();
                for (int k = 0; k < fetcherInfo.fetchers.length; ++k) {
                    if (fetcherInfo.fetchers[k] == currentThread3) {
                        fetcherInfo.fetchers[k] = null;
                        --fetcherInfo.numFetchers;
                    }
                }
            }
        }
    }
    
    private void fetchloop() {
        Thread currentThread = Thread.currentThread();
        while (isFetcher(currentThread)) {
            Thread.interrupted();
            currentThread.setPriority(8);
            ImageFetchable nextImage = nextImage();
            if (nextImage == null) {
                return;
            }
            try {
                nextImage.doFetch();
            }
            catch (Exception ex) {
                System.err.println("Uncaught error fetching image:");
                ex.printStackTrace();
            }
            stoppingAnimation(currentThread);
        }
    }
    
    static void startingAnimation() {
        FetcherInfo fetcherInfo = FetcherInfo.getFetcherInfo();
        Thread currentThread = Thread.currentThread();
        synchronized (fetcherInfo.waitList) {
            int i = 0;
            while (i < fetcherInfo.fetchers.length) {
                if (fetcherInfo.fetchers[i] == currentThread) {
                    fetcherInfo.fetchers[i] = null;
                    --fetcherInfo.numFetchers;
                    currentThread.setName("Image Animator " + i);
                    if (fetcherInfo.waitList.size() > fetcherInfo.numWaiting) {
                        createFetchers(fetcherInfo);
                    }
                    return;
                }
                else {
                    ++i;
                }
            }
        }
        currentThread.setPriority(2);
        currentThread.setName("Image Animator");
    }
    
    private static void stoppingAnimation(Thread thread) {
        FetcherInfo fetcherInfo = FetcherInfo.getFetcherInfo();
        synchronized (fetcherInfo.waitList) {
            int i = -1;
            int j = 0;
            while (j < fetcherInfo.fetchers.length) {
                if (fetcherInfo.fetchers[j] == thread) {
                    return;
                }
                if (fetcherInfo.fetchers[j] == null) {
                    i = j;
                }
                ++j;
            }
            if (i >= 0) {
                fetcherInfo.fetchers[i] = thread;
                ++fetcherInfo.numFetchers;
                thread.setName("Image Fetcher " + i);
                return;
            }
        }
    }
    
    private static void createFetchers(final FetcherInfo fetcherInfo) {
        AppContext appContext = AppContext.getAppContext();
        final ThreadGroup threadGroup = appContext.getThreadGroup();
        try {
            if (threadGroup.getParent() != null) {
            }
            else {
                ThreadGroup threadGroup2 = Thread.currentThread().getThreadGroup();
                for (ThreadGroup threadGroup3 = threadGroup2.getParent(); threadGroup3 != null && threadGroup3.getParent() != null; threadGroup3 = threadGroup2.getParent()) {
                    threadGroup2 = threadGroup3;
                }
                threadGroup = threadGroup2;
            }
        }
        catch (SecurityException ex) {
            threadGroup = appContext.getThreadGroup();
        }
        AccessController.doPrivileged(new PrivilegedAction() {
            @Override
            public Object run() {
                for (int i = 0; i < fetcherInfo.fetchers.length; ++i) {
                    if (fetcherInfo.fetchers[i] == null) {
                        ImageFetcher o = new ImageFetcher(threadGroup, i);
                        try {
                            o.start();
                            fetcherInfo.fetchers[i] = o;
                            FetcherInfo val$info = fetcherInfo;
                            ++info.numFetchers;
                            break;
                        }
                        catch (Error error) {}
                    }
                }
                return null;
            }
        });
    }
    
    ImageFetcher(ThreadGroup threadGroup, int n, ImageFetcher.1 privilegedAction) {
        this(threadGroup, n);
    }
}
